/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of Dekko email client for Ubuntu Devices/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import QtQuick 2.0
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import Ubuntu.Components.Popups 1.0
import Ubuntu.Content 0.1
import U1db 1.0 as U1db
import "../Dialogs"
import "../Utils/Utils.js" as Utils
import TrojitaCore 0.1
import DekkoCore 0.1

UbuntuListView {
    id: composer
    property alias replyingToMessage: recipientField.replyMode
    property alias replyMode: msgReplyMode.replyMode
    property QtObject draftsDatabase
    property var docId
    property int replyUid
    property string replyMboxName
    property var toAddresses
    property var ccAddresses
    property var bccAddresses
    property alias subject: composerUtils.subject
    property alias message: composerUtils.message
    property bool messageStarted: submissionManager.recipientsModel.count > 0
                                  || recipientField.text
                                  || subjectField.text
    property bool isEditingDraft: false
    property alias _recipientField: recipientField
    property alias identityModel: identitySelector.model

    signal submissionSuceeded

    model: composerItemModel
    clip: true
    anchors {
        fill: parent
        bottomMargin: units.gu(1)
    }
    contentHeight: composerItemModel.height + units.gu(10)

    function loseFocus() {
        recipientField.hasFocus = false
        subjectField.hasFocus = false
        messageField.hasFocus = false
    }

    function resetComposer() {
        submissionManager.recipientsModel.clear()
        subjectField.text = ""
        messageField.text = ""
        recipientField.resetSelector()
        recipientField.placeholderText = i18n.tr("Enter recipient address")
        recipientField.text = ""
        docId = ""
        isEditingDraft = false
        if (replyMode !== ReplyMode.REPLY_NONE) {
            subjectField.text = composerUtils.subject
            if (dekko.currentAccountName) {
                messageField.text = composerUtils.message
            } else {
                messageField.text = ""
            }
            submissionManager.recipientsModel.buildRecipientList(replyMode, imapAccess.oneMessageModel.messageIndex)
        }
        // manually reset this as clearing the model doesn't update the placeholder
        signatureLabel.text = getSignature()

    }


    function sendEmail() {
        submissionManager.setAccount(currentAccount.accountName)
        loseFocus()
        // lets do a sanity check and see if there is an address not currently in the list model
        if (recipientField.text) {
            recipientField.createAddressObject(
                        recipientField.formatAddresses(recipientField.text),
                        recipientField.getRecipientKind()
                        )
        }
        switch (replyMode) {
        case ReplyMode.REPLY_NONE:
            break;
        case ReplyMode.REPLY_ALL:
        case ReplyMode.REPLY_LIST:
        case ReplyMode.REPLY_PRIVATE:
            submissionManager.inReplyTo = imapAccess.oneMessageModel.messageId;
            submissionManager.setReferences(imapAccess.oneMessageModel.replyReferences())
            submissionManager.setReplyingToMessage(composer.replyMboxName, composer.replyUid)
            break;
        }
        if (submissionManager.setupConnection()) {
            // if we fail building the message the messagebuildingfailed signal will raise the error dialog
            // so nothing more to do here.
            if (submissionManager.buildMessage(identitySelector.model.get(identitySelector.selectedIndex).formattedString,
                                               subjectField.text,
                                               messageField.text + getSignature())) {
                submissionManager.send();
                showStatus();
                console.log("SELECTED INDEX = " + identitySelector.selectedIndex)
            }
        }
    }

    function showStatus() {
        PopupUtils.open(statusDialog)
    }

    function saveNewDraft() {
        var toAddr = new Array()
        var ccAddr = new Array()
        var bccAddr = new Array()
        for (var i =0; i < submissionManager.recipientsModel.count; i++) {
            var kind = submissionManager.recipientsModel.get(i).recipientKind
            var addr = submissionManager.recipientsModel.get(i).recipientAddress
            switch (kind) {
            case Recipient.ADDRESS_TO:
                toAddr.push(addr)
                break
            case Recipient.ADDRESS_CC:
                ccAddr.push(addr)
                break
            case Recipient.ADDRESS_BCC:
                bccAddr.push(addr)
                break
            }
        }
        draftsDatabase.putDoc(
                    {
                        to: toAddr,
                        cc: ccAddr,
                        bcc: bccAddr,
                        recipientFieldText: recipientField.text,
                        subject: subjectField.text,
                        message: messageField.text,
                        date: Qt.formatDate(new Date(), "dd/MM/yyyy"),
                        timestamp: Qt.formatTime(new Date(), "hh:mm")
                    }
                    )
    }

    function openDraft(docId) {
        resetComposer()
        var doc = draftsDatabase.getDoc(docId)
        var hasRecipients = false;
        if (doc.to !== undefined) {
            recipientField.fromAddr = doc.to
            hasRecipients = true;
        }
        if (doc.cc !== undefined) {
            recipientField.ccAddr = doc.cc
            hasRecipients = true
        }
        if (doc.bcc !== undefined) {
            recipientField.bccAddr = doc.bcc
            hasRecipients = true
        }
        if (hasRecipients) {
            recipientField.buildReplyMessage()
        }
        recipientField.text = doc.recipientFieldText
        subjectField.text = doc.subject
        messageField.text = doc.message
        draftsPanel.close()
    }
   //TODO: FIXME!!!
    function updateDraft() {
        // let's just delete the current doc and save a new one
        draftsDatabase.deleteDoc(docId)
        saveNewDraft()
    }

    function getSignature() {
        /* we want to force two empty lines then sig seperator then the signature
          So we always want to try and have two clear lines above the signature seperator

          >\n (initial break from message quote if ReplyMode.REPLY_ALL)
          \n (1st line)
          \n (2nd line)
          -- \n (3rd line, don't forget signature seperator requires a single space padding after the '--')
          <signature> (followed by signature)
        */
        if (identitySelector.model.get(identitySelector.selectedIndex).signature) {
            var sigSetup = "\n\n-- \n"
            if (replyMode === ReplyMode.REPLY_ALL) {
                sigSetup = "\n" + sigSetup
            }
            return sigSetup  + identitySelector.model.get(identitySelector.selectedIndex).signature
        } else {
            console.log("No signature for identity. Leaving Empty")
            return ""
        }
    }

    function buildingFailed(msg) {
        PopupUtils.open(Qt.resolvedUrl("../Dialogs/InfoDialog.qml"), dekko, {
                            title: i18n.tr("Oops!"),
                            text: i18n.tr(msg),
                            buttonText: i18n.tr("Close")
                        }
                        );
    }
    // FIXME: this is a workaround for the mailbox list view composer
    // We have to wait for the identity model to get populated fully so we can
    // Set the signature label correctly and the identity selector has a selected index
    Connections {
        target: currentAccount.identityModel
        onCountChanged: {
            if (count > 0) {
                identityModel = []
                identitySelector.model = currentAccount.identityModel
                identitySelector.selectedIndex = 0
            }
        }
    }

    ReplyMode { id: msgReplyMode }

    ComposerUtils {
        id: composerUtils
        // force a new line here
        fromName: imapAccess.oneMessageModel ?
                      Utils.formatPlainArrayMailAddresses(imapAccess.oneMessageModel.from, true, false).toString() :
                      undefined
        date: imapAccess.oneMessageModel ? imapAccess.oneMessageModel.date : undefined
        onMessageChanged: {
            if (!bottomEdge.opened) {
                messageField.text = composerUtils.message + "\n"
            }
        }
        onSubjectChanged: subjectField.text = composerUtils.subject
    }

    VisualItemModel {
        id: composerItemModel
        ListItem.ValueSelector {
            id: messageTypeSelector
            visible: replyMode !== ReplyMode.REPLY_NONE
            height: visible ? __height : 0
            values: imapAccess.oneMessageModel.isListPost ? [
                                                                i18n.tr("Reply"),
                                                                i18n.tr("Reply All"),
                                                                i18n.tr("Reply List")
                                                            ] :
                                                            [
                                                                i18n.tr("Reply"),
                                                                i18n.tr("Reply All")
                                                            ]
            onSelectedIndexChanged: {
                switch(selectedIndex) {
                case 0:
                    replyMode = ReplyMode.REPLY_PRIVATE;
                    break;
                case 1:
                    replyMode = ReplyMode.REPLY_ALL;
                    break;
                case 2:
                    replyMode = ReplyMode.REPLY_LIST;
                    break;
                }
                submissionManager.recipientsModel.buildRecipientList(replyMode, imapAccess.oneMessageModel.messageIndex)
            }
        }
        ListItem.Header {
            text: i18n.tr("From")
        }
        ListItem.ItemSelector {
            id: identitySelector
            expanded: false
            multiSelection: false
            selected: true
            delegate: OptionSelectorDelegate { text: formattedString }
            model: imapAccess.imapModel ? currentAccount.identityModel : []
            onModelChanged: model.count ? console.log("COUNT IS: " + count + ". Selected index is: " + selectedIndex) : console.log("Model empty")
            onSelectedIndexChanged: signatureLabel.text = getSignature()
        }
        ListItem.Header {
            text: i18n.tr("Recipients")
        }

        RecipientField {
            id: recipientField
            width: composer.width
            placeholderText: submissionManager.recipientsModel.count > 0
                             ? i18n.tr("Enter another recipient address")
                             : i18n.tr("Enter recipient address")
        }
        ListItem.Header {
            text: i18n.tr("Subject")
        }
        SubjectField{
            id: subjectField
            width: composer.width
            placeholderText: i18n.tr("Enter Subject")
        }
        ListItem.Header {
            text: i18n.tr("Message")
        }
        MessageField {
            id: messageField
            width: composer.width
            placeholderText: i18n.tr("Enter Message")
            onHasActiveFocusChanged: {
                if (hasActiveFocus) {
                    positionViewAtIndex(5, ListView.Beginning)
                } else {
                    console.log("Message field lost active focus")
                }
            }
            // start with an initial negative value to be sure key events trigger from the off,
            // as the cursor's y position if text area is empty will be 0.0.
            property real positionY: -1.0
            // lets try our best to keep the cursor in the View
            // as we have autosize enabled and positionViewAtXX doesn't provide the means
            // for mid message editing without the cursor going off screen.
            // This quite a simple solution at the moment where we just use
            // key release events, as this is the only time we should update/re-position the view,
            // in case the user is moving around mid message, using cursorposition changed events
            // would cause the view to skip all over the place.
            onKeyPressed: {
                var p = getRectPos().y
                if (p === positionY) {
                    return
                } else {
                    composer.contentY = contentY + p
                }
                positionY = p
                composer.returnToBounds()
            }
        }

        Item {
            width: parent.width
            height: signatureLabel.height
            Label {
                id: signatureLabel
                anchors {
                    left: parent.left
                    leftMargin: units.gu(2)
                    rightMargin: units.gu(2)
                    right: parent.right
                }
                text: (bottomEdge.opened && text === "") ? signatureLabel.text = getSignature() : ""
            }
        }

        Item {
            height: units.gu(1)
            width: composer.width
        }
        Column {
            width: parent.width
            Repeater {
                model: submissionManager.attachmentsModel
                delegate: ListItem.Subtitled {
                    text: caption
                    subText: mimeType + ", " + humanReadableSize
                    removable: true
                    confirmRemoval: true
                    iconFrame: false
                    iconName: mimeTypeIconName + "-symbolic"
                    fallbackIconName: "empty-symbolic"
                    showDivider: false
                    onItemRemoved: {
                        submissionManager.attachmentsModel.removeAttachmentAtIndex(index);
                    }
                }
            }
        }
        // empty item to position view to on active focus
        // if messageField visible area is greater than flickable visible area
        Item {
            height: units.gu(5)
            width: composer.width
        }
    }

    Component {
        id: statusDialog
        SubmissionStatusDialog {
            id: submissionStatusDialog
            onCloseClicked: {
                if (succeeded) {
                    submissionSuceeded()
                    identitySelector.selectedIndex = 0
                    succeeded = false
                }
                PopupUtils.close(submissionStatusDialog)
            }
        }
    }

    Component.onCompleted: {

        smtpPasswordWatcher.reloadPassword(currentAccount.accountName, "smtp")
        submissionManager.messageBuildFailed.connect(buildingFailed)
        submissionManager.passwordRequested.connect(function (user, host) {
            submissionManager.setPassword(smtpPasswordWatcher.password)
        });
        submissionManager.gotPassword.connect(function () { console.log("Password accepted.") })
    }
}
