/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of Dekko email client for Ubuntu Devices/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import QtQuick 2.2
import Ubuntu.Components 1.1
import Ubuntu.Components.ListItems 1.0 as ListItem
import TrojitaCore 0.1
import DekkoCore 0.1
import "../Components"

MultipleSelectionListView {
    id: root

    //-----------------------------------
    // PROPERTIES
    //-----------------------------------
    property alias hasScrollbar: listViewScrollBar.visible
    property var _currentSwipedItem: null

    //-----------------------------------
    // SIGNALS
    //-----------------------------------
    signal messageSelected(int msgUid)

    //-----------------------------------
    // SLOTS
    //-----------------------------------
    onMessageSelected: internal.openMessage(msgUid)
    onSelectionDone: {
        for (var i=0; i < items.count; i++) {
            accountsListModel.removeRow(items.get(i).index)
        }
    }

    onIsInSelectionModeChanged: {
        if (isInSelectionMode && _currentSwipedItem) {
            _currentSwipedItem.resetSwipe()
            _currentSwipedItem = null
        }
    }
    // Fire the sort timer
    Component.onCompleted: threadingMsgListModelSortTimer.start()
    // On destruction we reset the mailboxlist and msglistmodel
    // to prepare for another mailbox being opened.
    Component.onDestruction: dekko.resetMailboxList()

    //----------------------------------
    // FUNCTIONS
    //----------------------------------
    function resetSwipe() {
        if (_currentSwipedItem) {
            _currentSwipedItem.resetSwipe()
            _currentSwipedItem = null
        }
    }

    function _updateSwipeState(item) {
        if (item.swipping) {
            return
        }

        if (item.swipeState !== "Normal") {
            if (_currentSwipedItem !== item) {
                if (_currentSwipedItem) {
                    _currentSwipedItem.resetSwipe()
                }
                _currentSwipedItem = item
            }
        } else if (item.swipeState !== "Normal" && _currentSwipedItem === item) {
            _currentSwipedItem = null
        }
    }

    MessageListModel {
        id: messageListModel
        model: imapAccess.imapModel
        sortOrder: Qt.DescendingOrder
        mailbox: messageListPage.title
    }

    Component {
        id: sectionComponent
        Label {
            id: sectionLabel
            anchors {
                left: parent.left
                leftMargin: units.gu(2)
                right: parent.right
                rightMargin: units.gu(2)
            }
            text: section
            height: units.gu(5)
            fontSize: "medium"
            font.weight: Font.DemiBold
            verticalAlignment: Text.AlignVCenter
            ListItem.ThinDivider {
                anchors {
                    left: parent.left
                    right: parent.right
                    bottom: parent.bottom
                }
            }
        }
    }

    //----------------------------------
    // OBJECT PROPERTIES
    //----------------------------------
    // FIXME: the MessageListModel doesn't work well with sections
    // disabling for now as it's just eye-candy and can be sorted at a later time
//    section.property: "fuzzyDate"
//    section.delegate: sectionComponent

    listModel: currentAccount.devModeEnableThreading ? imapAccess.threadingMsgListModel : messageListModel
    displaced: Transition {
        UbuntuNumberAnimation {
            property: "y"
        }
    }
    remove: Transition {
        ParallelAnimation {
            UbuntuNumberAnimation {
                property: "height"
                to: 0
            }

            UbuntuNumberAnimation {
                properties: "opacity"
                to: 0
            }
            ScriptAction {
                script: {
                    resetSwipe()
                }
            }
        }
    }
    listDelegate: MessageListDelegate {
        id: delegate
        anchors {
            left: parent.left
            right: parent.right
        }
        onMarkMessageDeleted: imapAccess.markMessageDeleted(imapAccess.deproxifiedIndex(root.model.modelIndex(model.index)),
                                                            !model.isMarkedDeleted)
        onMarkMessageFlagged: flaggedAction.trigger()

        onItemClicked: {
            if (model.hasModelChildren) {
                root.model.rootIndex = root.model.modelIndex(index)
            } else {
                messageSelected(messageUid)
            }
        }
//            messageSelected(messageUid)
        selected: messagesListView.isSelected(delegate)
        selectionMode: messagesListView.isInSelectionMode
        isFirst: model.index === 0
        locked: !dekko.networkAvailable
        active: ListView.isCurrentItem

        leftSideAction: Action {
            id: deleteMsg
            iconName: "delete"
            text: "Delete"
            onTriggered: handleMarkMessageForDeletionByUserPrefs()
        }

        rightSideActions: [
            Action {
                id: flaggedAction
                text: isMarkedFlagged ? i18n.tr("Un-mark") : i18n.tr("Mark")
                iconName: isMarkedFlagged ? "favorite-unselected" : "favorite-selected"
                onTriggered: imapAccess.markMessageFlagged(imapAccess.deproxifiedIndex(root.model.modelIndex(model.index)),
                                                           !model.isMarkedFlagged)
            },
            Action {
                id: readAction
                text: isMarkedRead ? i18n.tr("Mark un-read") : i18n.tr("Mark read")
                iconSource: isMarkedRead ? Qt.resolvedUrl("../../icons/actions/email.svg") : Qt.resolvedUrl("../../icons/actions/email-read.svg")
                onTriggered: imapAccess.markMessageRead(imapAccess.deproxifiedIndex(root.model.modelIndex(model.index)), !isMarkedRead)
            }

        ]
    }

    PullToRefresh {
        refreshing: imapAccess.visibleTasksModel ? imapAccess.visibleTasksModel.hasVisibleTasks : false
        onRefresh: {
            if (dekko.networkAvailable) {
                imapAccess.imapModel.resyncMailbox(imapAccess.msgListModel.currentMailbox())
            }
        }
    }

    //-----------------------------------
    // PRIVATE PROPERTIES & FUNCTIONS
    //-----------------------------------
    QtObject {
        id: internal

        function sortThreadingListModelByUserSortPrefs() {
            // We have to force the sort order each time the view is created
            // but force it using the current sort order already in the model
            if (imapAccess.hasThreadingCapabilities) {
                imapAccess.threadingMsgListModel.setUserWantsThreading(true)
                // Initially set the sort order to descending, if the sort order
                // then gets changed from message list page toolbar, we respect that
                // change and use the specified sort order from then onwards
                imapAccess.threadingMsgListModel.setUserSearchingSortingPreference(
                            [],
                            ThreadingMsgListModel.SORT_DATE,
                            imapAccess.threadingMsgListModel.currentSortOrder()
                            )
            } else {
                imapAccess.threadingMsgListModel.setUserWantsThreading(false)
                imapAccess.threadingMsgListModel.setUserSearchingSortingPreference(
                            [],
                            ThreadingMsgListModel.SORT_NONE,
                            imapAccess.threadingMsgListModel.currentSortOrder()
                            )
            }
        }

        function openMessage(msgUid) {

            imapAccess.openMessage(mailboxList.currentMailboxLong, msgUid)
            pageStack.push(Qt.resolvedUrl("../MessageView/OneMessagePage.qml"),
                           {
                               mailbox: mailboxList.currentMailboxLong,
                               url: imapAccess.oneMessageModel.mainPartUrl,
                               replyUid: msgUid,
                               replyMboxName: mailboxList.currentMailboxLong
                           }
                           )
        }
    }

    //---------------------------------
    // COMPONENTS
    //---------------------------------
    Timer {
        id: threadingMsgListModelSortTimer
        repeat: false
        interval: 60
        onTriggered: internal.sortThreadingListModelByUserSortPrefs()
    }

    Scrollbar {
        id: listViewScrollBar
        visible: false
        flickableItem: root
        align: Qt.AlignTrailing
    }
}
