/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of Dekko email client for Ubuntu Touch/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
import QtQuick 2.2
import Ubuntu.Components 1.1
import Ubuntu.Components.Popups 1.0
import Ubuntu.Components.ListItems 1.0 as ListItems
// Disabling until the connectivity api works properly.
// This should stop the bad reviews from people who cannot be bothered to update
// their phones
//import Ubuntu.Connectivity 1.0
import DekkoCore 0.1
import TrojitaCore 0.1
import "./AccountsView"
import "./Dialogs"
import "./MailboxView"
import "./Utils"

MainView{
    id: dekko

    //-----------------------------------
    // PROPERTIES
    //-----------------------------------
    // Property holding the name of the currently open account
    property string currentAccountName // TODO: this should probably be currentAccountId as it's a Quuid
    // We can't make this one read-only as the password dialog needs to set it after password entered
    // NOTE: please DO NOT set this to false unless it is directly after entered password
    //       It will cause the authentication to loop out of control and get a DoS, bad error response
    property alias authenticateHasFailed: internal.authenticateHasFailed
    readonly property alias authenticateRequested: internal.authenticateRequested
    readonly property alias networkErrorRecieved: internal.networkErrorRecieved
    readonly property bool networkAvailable: !internal.networkOffline

    //-----------------------------------
    // SIGNALS
    //-----------------------------------
    signal reconnectAttemptScheduled(int timeout)
    signal showAccounts()
    signal openCurrentAccount()
    signal resetMailboxList()
    signal autoConfigComplete(var config, string username, string password)
    signal manualConfig(string username, string password)
    signal closeImapConnection()

    //----------------------------------
    // SLOTS
    //----------------------------------
    onCurrentAccountNameChanged: {
        console.log("[LOG] account name changed.")
        if (currentAccount.accountName === currentAccountName) {
            return;
        }
        console.log("[LOG] reloading settings for account " + currentAccountName)
        currentAccount.reloadSettings(currentAccountName)
        imapPasswordWatcher.reloadPassword(currentAccountName, "imap")
        smtpPasswordWatcher.reloadPassword(currentAccountName, "smtp")
    }
    onReconnectAttemptScheduled: console.log("Reconnect in: " + timeout)
    onShowAccounts: showAccountsAction.trigger()
    onOpenCurrentAccount: openCurrentAccountAction.trigger()
    onResetMailboxList: internal.resetMessageListView()
    onCloseImapConnection: internal.closeImapConnection()

    //----------------------------------
    // OBJECT PROPERTIES
    //----------------------------------
    objectName: "dekko" // object name for functional testing
    applicationName: "com.ubuntu.developer.dpniel.dekko"
    automaticOrientation: false
    anchorToKeyboard: true // ensure page doesn't end up behind the OSK
    // Random values to simulate a phone sized window on desktop
    // THis get's overriden on device
    width: units.gu(45); height: units.gu(71)
    useDeprecatedToolbar: false // disable use of the "old style" toolbar

    //--------------------------------
    // ACTIONS
    //--------------------------------
    actions: [
        Action {
            id: openCurrentAccountAction
            onTriggered: internal.openCurrentAccount()
        },
        Action {
            id: showAccountsAction
            onTriggered: internal.showAccounts()
        }
    ]

    //--------------------------------
    // PRIVATE PROPERTIES & FUNCTIONS
    //--------------------------------
    QtObject {
        id: internal

        property bool networkOffline: true
        property bool authenticateRequested: false
        property bool authenticateHasFailed: false
        property bool networkErrorRecieved: false

        function openImapConnection() {
            console.log("OPEN IMAP CONNECTION")
            pageStack.clear()
            imapPasswordWatcher.readingDone.connect(continueAuthentication)
            imapPasswordWatcher.readingFailed.connect(passwordReadingFailed)
            imapAccess.doConnect()
        }

        function closeImapConnection() {
            console.log("CLOSE IMAP CONNECTION")
            imapPasswordWatcher.readingDone.disconnect(continueAuthentication)
            imapPasswordWatcher.readingFailed.disconnect(passwordReadingFailed)
            imapAccess.reset()
        }

        function showImapError(message) {
            console.log("[IMAP ERROR] Imap error recieved with message: " + message)
            showInfoMessageToUser(i18n.tr("Error"), message)
        }
        function showNetworkError(message) {

            console.log("[NETWORK ERROR] Network error recieved with message " + message)
            // We only wat to show this if
            // 1) We are not trying to connect without an sslMode i.e first run while setting up connections
            // 2) the dialog isn't already visible, and further reconnect attempts are firing the networkError signal
            if (imapAccess.sslMode && !networkErrorRecieved) {
                PopupUtils.open(Qt.resolvedUrl("./Dialogs/InfoDialog.qml"), dekko, {
                                    networkError: true,
                                    title: i18n.tr("Network Message"),
                                    text: message
                                })
                networkErrorRecieved = true
            }
        }
        function showImapAlert(message) {
            console.log("[IMAP ALERT] imap alert recieved with message: " + message)
            showInfoMessageToUser(i18n.tr("Server Message"), message)
        }
        function mboxDeletionSuccess(mbox) {
            showInfoMessageToUser(i18n.tr("Complete"), i18n.tr("Successfully deleted %1").arg(mbox))
        }
        function mboxDeletionFailed(mbox, msg) {
            showInfoMessageToUser(i18n.tr("Unsuccessful"), i18n.tr("Failed deleting %1: %1").arg(mbox).arg(msg))
        }
        function mboxSyncFailed(mbox, msg) {
            showInfoMessageToUser(i18n.tr("Syncing failed"), i18n.tr("Failed syncing %1: %1").arg(mbox).arg(msg))
        }

        function resetMessageListView() {
            console.log("[LOG] Message list model about to be reset")
            imapAccess.msgListModel.resetMe()
            mailboxList.mailBox = ""
        }

        function continueAuthentication() {
            // avoid getting stuck in a authenticate loop
            console.log("[LOG] Continuing authentication")
            console.log("Auth failed?: " + authenticateHasFailed)
            if (!authenticateHasFailed && authenticateRequested) {
                if (imapPasswordWatcher.didReadOk && imapPasswordWatcher.password) {
                    console.log("[LOG] Password found. Setting Imap model password")
                    imapAccess.imapModel.imapPassword = imapPasswordWatcher.password
                } else {
                    console.log("[LOG] No password saved. Opening password dialog....")
                    PopupUtils.open(passwordDialogComponent)
                }
                authenticateRequested = false
            } else {
                console.log("Oh no authRequested signal yet. Lets continue and wait for an *actual* signal")
            }
        }

        function passwordReadingFailed(message) {
            console.log("[LOG] Password Reading Failed:" + message)
            authAttemptFailed(message)
            PopupUtils.open(passwordDialogComponent)
        }

        function authAttemptFailed(message) {
            console.log("[IMAP ERROR] Authentication failed with message: " + message)
            authenticateHasFailed = true
            PopupUtils.open(passwordDialogComponent)
            passwordInput.title = i18n.tr("<font color='#333333'>Authentication Error</font>")
            passwordInput.authMessage = message
            passwordInput.settingsMessage = i18n.tr("Try re-entering account password or click cancel to go to Accounts")
        }

        function showInfoMessageToUser(title, message) {
            PopupUtils.open(Qt.resolvedUrl("./Dialogs/InfoDialog.qml"), dekko, {
                                title: title,
                                text: message
                            })
        }

        function connectModels() {
            console.log("[LOG] Connecting model signals")
            imapAccess.imapModel.imapError.connect(showImapError)
            imapAccess.imapModel.networkError.connect(showNetworkError)
            imapAccess.imapModel.alertReceived.connect(showImapAlert)
            imapAccess.imapModel.authRequested.connect(authRequested)
            imapAccess.imapModel.authAttemptFailed.connect(authAttemptFailed)
            imapAccess.imapModel.networkPolicyOffline.connect(function() {networkOffline = true})
            imapAccess.imapModel.networkPolicyOnline.connect(function() {networkOffline = false})
            imapAccess.imapModel.networkPolicyExpensive.connect(function() {networkOffline = false})
            imapAccess.imapModel.mailboxDeletionSucceded.connect(mboxDeletionSuccess)
            imapAccess.imapModel.mailboxDeletionFailed.connect(mboxDeletionFailed)
            imapAccess.imapModel.capabilitiesUpdated.connect(isthreadingAvailable)
            imapAccess.imapModel.mailboxSyncFailed.connect(mboxSyncFailed)
            imapAccess.msgListModel.mailboxChanged.connect(mailboxList.pushMessageListView)
            imapAccess.networkWatcher.reconnectAttemptScheduled.connect(reconnectAttemptScheduled)
        }
        function disconnectModels() {
            console.log("[LOG] disconnecting model signals")
            imapAccess.imapModel.imapError.disconnect(showImapError)
            imapAccess.imapModel.networkError.disconnect(showNetworkError)
            imapAccess.imapModel.alertReceived.disconnect(showImapAlert)
            imapAccess.imapModel.authRequested.disconnect(authRequested)
            imapAccess.imapModel.authAttemptFailed.disconnect(authAttemptFailed)
            imapAccess.imapModel.networkPolicyOffline.disconnect(function() {networkOffline = true})
            imapAccess.imapModel.networkPolicyOnline.disconnect(function() {networkOffline = false})
            imapAccess.imapModel.networkPolicyExpensive.disconnect(function() {networkOffline = false})
            imapAccess.imapModel.mailboxDeletionSucceded.disconnect(mboxDeletionSuccess)
            imapAccess.imapModel.mailboxDeletionFailed.disconnect(mboxDeletionFailed)
            imapAccess.imapModel.mailboxSyncFailed.disconnect(mboxSyncFailed)
            imapAccess.msgListModel.mailboxChanged.disconnect(mailboxList.pushMessageListView)
            imapAccess.networkWatcher.reconnectAttemptScheduled.disconnect(reconnectAttemptScheduled)
        }

        function isthreadingAvailable() {
            // NOTE: Until threading view is actually implemented hasThreadingCapabilities is always going to return
            // false, therefore there is no change in behaviour just yet
            if (imapAccess.hasThreadingCapabilities) {
                console.log("HAS THREADING CAPABILITIES")
                imapAccess.threadingMsgListModel.setUserWantsThreading(true)
                // Initially set the sort order to descending, if the sort order
                // then gets changed from message list page toolbar, we respect that
                // change and use the specified sort order from then onwards
                imapAccess.threadingMsgListModel.setUserSearchingSortingPreference(
                            [],
                            ThreadingMsgListModel.SORT_DATE,
                            Qt.DescendingOrder
                            )
            } else {
                console.log("NO THREADING CAPABILITIES")
                imapAccess.threadingMsgListModel.setUserWantsThreading(false)
                imapAccess.threadingMsgListModel.setUserSearchingSortingPreference(
                            [],
                            ThreadingMsgListModel.SORT_NONE,
                            Qt.DescendingOrder
                            )
            }
        }

        function authRequested() {
            console.log("[LOG] Authentication requested by the imap server")
            authenticateRequested = true
            console.log("[LOG] Reloading password")
            imapPasswordWatcher.reloadPassword(currentAccountName, "imap")
        }

        function setupConnections() {
            // connect these before calling imapAccess.doConnect()
            console.log("[LOG] Setting up initial signal connections")
            imapAccess.checkSslPolicy.connect(function() {PopupUtils.open(sslSheetPage)})
            imapAccess.modelsChanged.connect(modelsChanged)
            imapAccess.accountReady.connect(openCurrentAccount)
        }

        function openCurrentAccount() {
            console.log("[LOG] Opening account connection")
            if (imapAccess.imapModel) {
                // disconnect the old connections
                console.log("[LOG] disconnecting old model connections")
                disconnectModels()
                imapAccess.reset()
            }
            console.log("[LOG] Connecting .............................")
            openImapConnection()
            connectModels()
        }

        function settingsChanged() {
            console.log("[LOG] Settings changed")
            // when settings are changed we want to unload the mailboxview model
            mailboxList.model = null
        }

        function modelsChanged() {
            console.log("[LOG] Models changed. We need to reset the message list view now")
            mailboxList.model = imapAccess.mailboxModel
            mailboxList.visible = false
            resetMessageListView()
            showHome()
        }

        function showAccounts() {
            console.log("[LOG] Opening accounts page")
            if (imapAccess.imapModel) {
                closeImapConnection()
            }
            imapPasswordWatcher.clearCurrentPassword()
            smtpPasswordWatcher.clearCurrentPassword()
            pageStack.clear()
            pageStack.push(accountsPage)
        }

        function showHome() {
            console.log("[LOG] Show home()")
            pageStack.clear()
            pageStack.push(mailboxList)
            mailboxList.developerModeEnabled = currentAccount.developerModeEnabled
            mailboxList.nestingDepth = 0
            mailboxList.currentMailbox = ""
            mailboxList.currentMailboxLong = ""
            if (mailboxList.model)
                mailboxList.model.setOriginalRoot()
            mailboxList.reloadComposer()
        }
    }

    //--------------------------------
    // COMPONENTS / CHILD OBJECTS
    //-------------------------------
// TODO: Re-enable once connectivity api is fixed
//    DummyNetworkStatus {
//        id: dummyNetworkStatus
//    }
//    Connections {
//        id: networkStatus
//        target: imapAccess.isRunningOnMir ? NetworkingStatus : dummyNetworkStatus
//        onStatusChanged: {
//            if (status === NetworkingStatus.Offline) {
//                imapAccess.networkWatcher.setNetworkOffline()
//            }
//            if (status === NetworkingStatus.Online) {
//                if (NetworkingStatus.limitedBandwith) {
//                    imapAccess.networkWatcher.setNetworkExpensive()
//                } else {
//                    imapAccess.networkWatcher.setNetworkOnline()
//                }
//            }
//        }
//        onLimitedBandwithChanged: NetworkingStatus.limitedBandwith ? imapAccess.networkWatcher.setNetworkExpensive() : imapAccess.networkWatcher.setNetworkOnline()
//    }

    // TODO: Change this as sheets are going to be deprecated
    Component{
        id: sslSheetPage
        SslSheet {
            id: sslSheet
            title:  imapAccess.sslInfoTitle
            htmlText: imapAccess.sslInfoMessage
            onConfirmClicked: {
                imapAccess.setSslPolicy(true)
                // avoid calling showHome if it's already pushed
                if (pageStack.depth === 0) {
                    internal.showHome()
                }
            }
            onCancelClicked: imapAccess.setSslPolicy(false)
        }
    }
    // TODO: Change this as sheets are going to be deprecated
    Item {
        id: passwordInput
        // AWESOME!!, a bit of html tagging to the rescue
        property string title: i18n.tr("<font color='#333333'>Authentication Required</font>")
        property string authMessage
        property string settingsMessage
        Component {
            id: passwordDialogComponent
            PasswordInputSheet {
                id: passwordInputSheet
                title: passwordInput.title
                message: i18n.tr("Please provide IMAP password for user <b>%1</b> on <b>%2</b>:").arg(
                                     currentAccount.imapUsername).arg(currentAccount.imapServer)
                authErrorMessage: passwordInput.authMessage
                settingsMessage: passwordInput.settingsMessage
            }
        }
    }

    PageStack{
        id:pageStack
        onDepthChanged: console.log("[PAGESTACK] depth = " + depth)
        Component.onCompleted: {
            internal.setupConnections()
            if (accountsListModel.count === 1) {
                console.log("[LOG] Only one account in model, going straight to folder view")
                var accountName = accountsListModel.get(0).accountName
                console.log("Setting imap access account")
                imapAccess.setAccount(accountName)
                console.log("Settings submissionManager account")
                submissionManager.setAccount(accountName)
                smtpPasswordWatcher.reloadPassword(accountName, "smtp")
                currentAccountName = accountName
            } else {
                pageStack.push(accountsPage)
            }
        }

        AccountsPage {
            id: accountsPage
            visible: false
            onOpenAccount: {
                console.log("Setting ImapAccess account")
                imapAccess.setAccount(accountName)
                console.log("Settings sub manager account")
                submissionManager.setAccount(accountName)
                console.log("Reloading smtpPassword")
                smtpPasswordWatcher.reloadPassword(accountName, "smtp")
            }
        }

        // Access Granted show MailBox Lists
        MailboxListPage {
            id: mailboxList
            visible: false
            property string mailBox
            onMailboxSelected: {
                if (mailbox === mailBox) {
                    // There is no need to set the mailbox again
                    pushMessageListView()
                } else {
                    // Set a new mailbox and wait for the changed signal to push the view
                    mailBox = mailbox
                    imapAccess.msgListModel.setMailbox(mailbox)
                }
            }
            function pushMessageListView() {
                if (imapAccess.msgListModel.itemsValid) {
                    pageStack.push(Qt.resolvedUrl("./MessageListView/MessageListPage.qml"), { title: mailBox })
                }
            }
        }
    }
}
