/* Copyright (C) 2006 - 2014 Jan Kundrát <jkt@flaska.net>

   This file is part of the Trojita Qt IMAP e-mail client,
   http://trojita.flaska.net/

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef TROJITA_IMAPACCESS_H
#define TROJITA_IMAPACCESS_H

#include <QObject>
#include <QSslError>

#include "Common/ConnectionMethod.h"
#include "Imap/Model/MailboxModel.h"
#include "Imap/Model/Model.h"
#include "Imap/Model/MsgListModel.h"
#include "Imap/Model/NetworkWatcher.h"
#include "Imap/Model/OneMessageModel.h"
#include "Imap/Model/SubtreeModel.h"
#include "Imap/Model/Utils.h"
#include "Imap/Model/VisibleTasksModel.h"
#include "Imap/Model/ThreadingMsgListModel.h"
#include "Plugins/PluginManager.h"
#include "UiUtils/Formatting.h"
#include "UiUtils/PasswordWatcher.h"
#include "app/Accounts/Account.h"

class QNetworkAccessManager;
class QSettings;

namespace Dekko {
namespace Accounts {
class Account;
}
}

namespace Imap {

class ImapAccess : public QObject
{
    Q_OBJECT

    Q_PROPERTY(QObject *imapModel READ imapModel NOTIFY modelsChanged)
    Q_PROPERTY(QObject *mailboxModel READ mailboxModel NOTIFY modelsChanged)
    Q_PROPERTY(QObject *msgListModel READ msgListModel NOTIFY modelsChanged)
    Q_PROPERTY(QObject *threadingMsgListModel READ threadingMsgListModel NOTIFY modelsChanged)
    Q_PROPERTY(QObject *visibleTasksModel READ visibleTasksModel NOTIFY modelsChanged)
    Q_PROPERTY(QObject *oneMessageModel READ oneMessageModel NOTIFY modelsChanged)
    Q_PROPERTY(QObject *networkWatcher READ networkWatcher NOTIFY modelsChanged)
    Q_PROPERTY(QObject *msgQNAM READ msgQNAM NOTIFY modelsChanged)
    Q_PROPERTY(QString sslInfoTitle READ sslInfoTitle NOTIFY checkSslPolicy)
    Q_PROPERTY(QString sslInfoMessage READ sslInfoMessage NOTIFY checkSslPolicy)
    Q_PROPERTY(QStringList unfilteredCapabilities READ unfilteredCapabilities NOTIFY modelsChanged)
    Q_PROPERTY(UiUtils::PasswordWatcher *passwordWatcher READ passwordWatcher NOTIFY modelsChanged)
    Q_PROPERTY(bool hasThreadingCapabilities READ hasThreadingCapabilities NOTIFY modelsChanged)
    Q_PROPERTY(bool isRunningOnMir READ isRunningOnMir CONSTANT)
    Q_ENUMS(Imap::ImapAccess::ConnectionMethod)

public:
    ImapAccess(QObject *parent, Plugins::PluginManager *pluginManager, QSettings *settings);

    QObject *imapModel() const;
    QObject *mailboxModel() const;
    QObject *msgListModel() const;
    QObject *visibleTasksModel() const;
    QObject *oneMessageModel() const;
    QObject *networkWatcher() const;
    QObject *threadingMsgListModel() const;
    QObject *msgQNAM() const;
    UiUtils::PasswordWatcher *passwordWatcher() const;

    bool hasThreadingCapabilities() const;
    QStringList unfilteredCapabilities() const;
    Q_INVOKABLE void doConnect();

    QString sslInfoTitle() const;
    QString sslInfoMessage() const;
    UiUtils::Formatting::IconType sslInfoIcon() const;
    bool isRunningOnMir() const;

    Q_INVOKABLE void openMessage(const QString &mailbox, const uint uid);
    Q_INVOKABLE QString prettySize(const uint bytes) const;
    Q_INVOKABLE void setSslPolicy(bool accept);

    Q_INVOKABLE void nukeCache();

    Q_INVOKABLE QString mailboxListShortMailboxName() const;
    Q_INVOKABLE QString mailboxListMailboxName() const;

    Q_INVOKABLE QModelIndex deproxifiedIndex(const QModelIndex index);
    Q_INVOKABLE void markMessageDeleted(const QModelIndex &message, bool marked);
    Q_INVOKABLE void markMessageFlagged(const QModelIndex &message, bool marked);
    Q_INVOKABLE void markMessageRead(const QModelIndex &message, bool marked);
signals:
    void modelsChanged();
    void checkSslPolicy();
    void cacheError(const QString &message);
    void accountReady();

public slots:
    void alertReceived(const QString &message);
    void imapError(const QString &message);
    void networkError(const QString &message);
    void onCacheError(const QString &message);
    void slotLogged(uint parserId, const Common::LogMessage &message);
    void slotSslErrors(const QList<QSslCertificate> &sslCertificateChain, const QList<QSslError> &sslErrors);
    void setAccount(const QString &accountName);
    void reset();

private:
    Imap::Mailbox::Model *m_imapModel;
    Imap::Mailbox::MailboxModel *m_mailboxModel;
    Imap::Mailbox::SubtreeModelOfMailboxModel *m_mailboxSubtreeModel;
    Imap::Mailbox::MsgListModel *m_msgListModel;
    Imap::Mailbox::ThreadingMsgListModel *m_threadingMsgListModel;
    Imap::Mailbox::VisibleTasksModel *m_visibleTasksModel;
    Imap::Mailbox::OneMessageModel *m_oneMessageModel;
    Imap::Mailbox::NetworkWatcher *m_netWatcher;
    QNetworkAccessManager *m_msgQNAM;
    Plugins::PluginManager *m_pluginManager;
    UiUtils::PasswordWatcher *m_passwordWatcher;

    QList<QSslCertificate> m_sslChain;
    QList<QSslError> m_sslErrors;
    QString m_sslInfoTitle;
    QString m_sslInfoMessage;
    UiUtils::Formatting::IconType m_sslInfoIcon;

    QString m_accountName;
    QString m_cacheDir;
    Dekko::Accounts::Account *m_account;
    QSettings *m_settings;
};

}

#endif // TROJITA_IMAPACCESS_H
