/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of the Dekko

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "AccountsListModel.h"
#include <QSettings>
#include <QDebug>
#include <QFile>
#include <QUuid>
#include "AccountUtils.h"

namespace Dekko
{
namespace Accounts
{

AccountsListModel::AccountsListModel(QObject *parent, QSettings *settings) :
    QAbstractListModel(parent), m_settings(settings)
{
}

void AccountsListModel::reloadAccounts()
{
    if (rowCount() > 0) {
        clear();
    }
    qDebug() << m_settings->fileName();
    if (QFile::exists(m_settings->fileName())) {
        AccountUtils::checkSettingsFilePermissions(m_settings);
    } else {
        qDebug("Account settings don't exist yet no need to check the permissions");
    }
    qDebug() << "AccountListModel: reloading accounts";
    QStringList groups;
    qDebug() << "AccountListModel: Current settings group: " << m_settings->group();
    m_settings->beginGroup("Accounts");
    qDebug() << "AccountListModel: Current settings group should now be Accounts: " << m_settings->group();
    groups = m_settings->childGroups();
    qDebug() << "AccountListModel: Child groups are: " << groups;
    m_settings->endGroup();
    Q_FOREACH(const QString &group, groups) {
        qDebug() << "AccountListModel: Creating account object for account: " << group;
        Account *account = new Account(this);
        qDebug() << "AccountListModel: Reloading settings for account";
        account->reloadSettings(m_settings, group);
        qDebug() << "AccountListModel: Appending account to model";
        appendRow(account);
        qDebug() << "AccountListModel: Append successful";
    }

}

QHash<int, QByteArray> AccountsListModel::roleNames() const
{
    QHash<int, QByteArray> roles;
    roles[AccountNameRole] = "accountName";
    roles[AccountDescriptionRole] = "description";
    roles[AccountImagePathRole] = "accountImage";
    roles[AccountUserNameRole] = "imapUsername";
    roles[AccountImapServerRole] = "imapServer";
    roles[AccountImapPortRole] = "imapPort";
    roles[AccountImapEncryptRole] = "imapEncryptMethod";
    roles[AccountSubmissionPortRole] = "submissionPort";
    roles[AccountSubmissionServerRole] = "submissionServer";
    roles[AccountSubmissionUserNameRole] = "submissionUsername";
    return roles;
}

QVariant AccountsListModel::data(const QModelIndex &index, int role) const
{
    if (index.row() < 0 || index.row() >= m_accounts.size()) {
        return QVariant();
    }
    switch (role) {
    case AccountNameRole:
        return m_accounts[index.row()]->accountName();
    case AccountDescriptionRole:
        return m_accounts[index.row()]->description();
    case AccountImagePathRole:
        return m_accounts[index.row()]->accountImage();
    case AccountUserNameRole:
        return m_accounts[index.row()]->imapUsername();
    case AccountImapServerRole:
        return m_accounts[index.row()]->imapServer();
    case AccountImapPortRole:
        return m_accounts[index.row()]->imapPort();
    case AccountImapEncryptRole:
        return m_accounts[index.row()]->imapSslMode();
    case AccountSubmissionPortRole:
        return m_accounts[index.row()]->submissionPort();
    case AccountSubmissionServerRole:
        return m_accounts[index.row()]->submissionServer();
    case AccountSubmissionUserNameRole:
        return m_accounts[index.row()]->submissionUsername();
    }
    return QVariant();
}

int AccountsListModel::rowCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent);
    return m_accounts.size();
}

QModelIndex AccountsListModel::indexFromItem(Account *account) const
{
    if (account != NULL) {
        for (int i = 0; i < m_accounts.size(); i++) {
            if (m_accounts.at(i) == account) {
                return index(i);
            }
        }
    }
    return QModelIndex();
}

bool AccountsListModel::removeRow(int row, const QModelIndex &index)
{
    if (row >= 0 || row < m_accounts.size()) {
        beginRemoveRows(index, row, row);
        Account *account = m_accounts.takeAt(row);
        account->deleteAccountSettings();
        delete account;
        account = 0;
        endRemoveRows();
        emit countChanged();
        return true;
    }
    return false;
}

bool AccountsListModel::removeRows(int row, int count, const QModelIndex &index)
{
    if (row >= 0 && count > 0 && (row + count) <= m_accounts.size())
    {
        beginRemoveRows(index, row, row + count - 1);
        Account *account = m_accounts.takeAt(row);
        account->deleteAccountSettings();
        Q_ASSERT(account);
        delete account;
        account = 0;
        endRemoveRows();
        emit countChanged();
        return true;
    }
    return false;
}

QVariant AccountsListModel::get(int index)
{
    if (index < 0 || index > m_accounts.size()) {
        return QVariant();
    }
    Account *account = m_accounts.at(index);
    QMap<QString, QVariant> accountData;
    accountData.insert(QString("accountName"), account->accountName());
    accountData.insert(QString("description"), account->description());
    return QVariant(accountData);
}

void AccountsListModel::appendRow(Account *account)
{
    appendRows(QList<Account *>() << account);
}

void AccountsListModel::appendRows(QList<Account *> &account)
{
    if (account.size() == 0) {
        return;
    }
    qDebug() << "Starting to insert rows";
    beginInsertRows(QModelIndex() , rowCount(), rowCount() + account.size() - 1);
    Q_FOREACH(Account *acc, account) {
        qDebug() << "Appending " << acc->accountName() << "to model";
//        connect(acc, SIGNAL(dataChanged()), this, SLOT(updateAccount()));
        m_accounts.append(acc);
    }
    endInsertRows();
    emit countChanged();
}

void AccountsListModel::insertRow(int row, Account *account)
{
    if (account == NULL) {
        return;
    }
    beginInsertRows(QModelIndex(), row, row);
    m_accounts.insert(row, account);
    endInsertRows();
    emit countChanged();
}

void AccountsListModel::clear()
{
    if (m_accounts.size() == 0) {
        return;
    }
    beginRemoveRows(QModelIndex(), 0, m_accounts.size());
    m_accounts.clear();
    endRemoveRows();
    emit countChanged();
}

}
}
