/* Copyright (C) 2014 Dan Chapman <dpniel@ubuntu.com>

   This file is part of Dekko mail client

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <QDebug>
#include <QQmlContext>
#include <QSettings>
#include <QtGui/QGuiApplication>
#include <QtQml/QtQml>
#include <QtQuick/QQuickView>
#include "AppVersion/SetCoreApplication.h"
#include "Common/Application.h"
#include "Common/MetaTypes.h"
#include "Common/SettingsNames.h"
#include "app/Composition/ReplyMode.h"
#include "app/Accounts/Account.h"
#include "app/Accounts/AccountsListModel.h"
#include "app/Accounts/SenderIdentity.h"
#include "app/Composition/SubmissionManager.h"
#include "app/Composition/ComposerUtils.h"
#include "app/MailConfig/MailConfigUtils.h"
#include "app/Models/Recipient.h"
#include "app/Models/RecipientListModel.h"
#include "app/Models/MessageListModel.h"
#include "app/PartWidget/SectionType.h"
#include "app/Utils/Path.h"
#include "app/Utils/PlainTextFormatter.h"
#include "Imap/Model/ImapAccess.h"
#include "Imap/Model/ThreadingMsgListModel.h"
#include "MSA/Account.h"
#include "Plugins/PluginManager.h"
#include "static_plugins.h"
#include "UiUtils/PartWidgetFactory.h"

QQuickView *qQuickViewer;

static void registerDekkoTypes(const char* uri, int major = 0, int minor = 1)
{
    // @uri DekkoCore
    qmlRegisterType<QSettings>(uri, major, minor, "QSettings");
    qmlRegisterType<Dekko::ReplyMode>(uri, major, minor, "ReplyMode");
    qmlRegisterType<Dekko::ComposerUtils>(uri, major, minor, "ComposerUtils");
    qmlRegisterType<Dekko::Models::Recipient>(uri, major, minor, "Recipient");
    qmlRegisterType<Dekko::Models::RecipientListModel>(uri, major, minor, "RecipientListModel");
    qmlRegisterType<Dekko::Accounts::Account>(uri, major, minor, "Account");
    qmlRegisterType<Dekko::Accounts::SenderIdentity>(uri, major, minor, "SenderIdentity");
    qmlRegisterType<Dekko::Models::RecipientListModel>(uri, major, minor, "RecipientListModel");
    qmlRegisterType<Dekko::Models::MessageListModel>(uri, major, minor, "MessageListModel");
    qmlRegisterType<Dekko::Utils::PlainTextFormatter>(uri, major, minor, "PlainTextFormatter");
    qmlRegisterUncreatableType<Dekko::ComposerUtils>(uri, major, minor, "RecipientKind",
                                                            QLatin1String("Can't create recipientkind from qml"));
    qmlRegisterUncreatableType<Dekko::PartWidget::SectionType>(uri, major, minor, "SectionType",
                                                            QLatin1String("Can't create sectiontype from qml"));
}

static void registerTrojitaCoreTypes(const char *uri, int major = 0, int minor = 1)
{
    // @uri TrojitaCore
    qmlRegisterUncreatableType<Imap::Mailbox::ThreadingMsgListModel>(uri, major, minor, "ThreadingMsgListModel",
            QLatin1String("ThreadingMsgListModel can only be created from the C++ code.\
                          Use ImapAccess if you need access to an instance."));
    qmlRegisterSingletonType<UiUtils::Formatting>(uri, major, minor, "UiFormatting", UiUtils::Formatting::factory);
    qmlRegisterUncreatableType<Composer::Submission>(uri, major, minor, "SubmissionMethod",
            QLatin1String("Composer::Submission::Method can be only created from the C++ code."));
    qmlRegisterUncreatableType<UiUtils::PasswordWatcher>(uri, major, minor, "PasswordWatcher",
            QLatin1String("PasswordWatcher can only be created from the C++ code. Use ImapAccess\
                          if you need access to an instance."));
}

int main(int argc, char *argv[])
{
    QGuiApplication app(argc, argv);

//    QDir::addSearchPath("icons", QDir::currentPath("/icons"));

    qQuickViewer = new QQuickView();
    qQuickViewer->setResizeMode(QQuickView::SizeRootObjectToView);
    app.addLibraryPath(QCoreApplication::applicationDirPath());

    // let's first check all standard locations
    QString qmlFile = Dekko::findQmlFile(QString::fromLatin1("qml/main.qml"));

    Common::registerMetaTypes();
    Common::Application::name = QString::fromLatin1("com.ubuntu.developer.dpniel.dekko");
    Common::Application::organization = QString::fromLatin1("com.ubuntu.developer.dpniel.dekko");
    AppVersion::setVersion();
    AppVersion::setCoreApplicationData();

    QSettings s;

    Plugins::PluginManager pluginManager(0, &s, Common::SettingsNames::addressbookPlugin,
                                         Common::SettingsNames::passwordPlugin);
    Imap::ImapAccess imapAccess(0, &pluginManager, &s);
    qQuickViewer->engine()->rootContext()->setContextProperty(QLatin1String("imapAccess"), &imapAccess);
    Dekko::SubmissionManager submissionManager(0, &imapAccess, &s);
    qQuickViewer->engine()->rootContext()->setContextProperty(QLatin1String("submissionManager"), &submissionManager);
    Dekko::Accounts::Account currentAccount(0);
    currentAccount.setSettings(&s);
    currentAccount.reset();
    qQuickViewer->engine()->rootContext()->setContextProperty(QLatin1String("currentAccount"), &currentAccount);
    Dekko::Accounts::AccountsListModel accountsListModel(0, &s);
    accountsListModel.reloadAccounts();
    qQuickViewer->engine()->rootContext()->setContextProperty(QLatin1String("accountsListModel"), &accountsListModel);
    Dekko::MailConfigUtils mailConfig;
    qQuickViewer->engine()->rootContext()->setContextProperty(QLatin1String("mailConfig"), &mailConfig);
    qQuickViewer->engine()->rootContext()->setContextProperty(QLatin1String("settingsFile"), &s);
    UiUtils::PasswordWatcher imapPasswordWatcher(0, &pluginManager);
    qQuickViewer->engine()->rootContext()->setContextProperty(QLatin1String("imapPasswordWatcher"),
                                                              &imapPasswordWatcher);
    UiUtils::PasswordWatcher smtpPasswordWatcher(0, &pluginManager);
    qQuickViewer->engine()->rootContext()->setContextProperty(QLatin1String("smtpPasswordWatcher"),
                                                              &smtpPasswordWatcher);

    UiUtils::PartWidgetFactory partWidgetFactory(new Imap::Network::MsgPartNetAccessManager());
    qQuickViewer->engine()->rootContext()->setContextProperty(QLatin1String("partWidgetFactory"), &partWidgetFactory);

    QUrl addressbookStoreLocation(QString::fromLatin1(
            "file://%1/.local/share/com.ubuntu.developer.dpniel.dekko/addressbook.vcard")
            .arg(QDir::homePath()));
    qQuickViewer->engine()->rootContext()->setContextProperty(QLatin1String("addressbookStoreLocation"), QVariant::fromValue(addressbookStoreLocation));

    registerDekkoTypes("DekkoCore");
    registerTrojitaCoreTypes("TrojitaCore");


    qQuickViewer->setTitle(QObject::trUtf8("Dekko"));
    qQuickViewer->setSource(QUrl::fromLocalFile(qmlFile));
    qQuickViewer->show();
    return app.exec();
}
